#include <jni.h>
#include <android/log.h>
#include "platform/android/jni/JniHelper.h"

// _BQ_TMPL_CONFIGURE_INCLUDE_BEGIN
// _BQ_TMPL_CONFIGURE_INCLUDE_SNSAUTH_BEGIN
#include "Common/UserData/UserDataBridge.h"
#include "Library/UtilityForSakura.h"
#include "Scene/ChangingMobileModel/SnsMobileDataMigration.h"
#include "Scene/ChangingMobileModel/SnsMobileDataMigrationListener.h"
#include "Scene/ChangingMobileModel/BNID/BNIDLinkResultChecker.h"
// _BQ_TMPL_CONFIGURE_INCLUDE_SNSAUTH_END
// _BQ_TMPL_CONFIGURE_INCLUDE_END
#include "AppConfigure.h"

extern "C"
{
void javaHashMapToStlStringStringMap(JNIEnv *env, jobject hashMap, std::map<std::string, std::string>& mapOut) {
    // Get the Map's entry Set.
    jclass mapClass = env->FindClass("java/util/Map");
    if (mapClass == NULL) {
        return;
    }
    jmethodID entrySet =
            env->GetMethodID(mapClass, "entrySet", "()Ljava/util/Set;");
    if (entrySet == NULL) {
        return;
    }
    jobject set = env->CallObjectMethod(hashMap, entrySet);
    if (set == NULL) {
        return;
    }
    // Obtain an iterator over the Set
    jclass setClass = env->FindClass("java/util/Set");
    if (setClass == NULL) {
        return;
    }
    jmethodID iterator =
            env->GetMethodID(setClass, "iterator", "()Ljava/util/Iterator;");
    if (iterator == NULL) {
        return;
    }
    jobject iter = env->CallObjectMethod(set, iterator);
    if (iter == NULL) {
        return;
    }
    // Get the Iterator method IDs
    jclass iteratorClass = env->FindClass("java/util/Iterator");
    if (iteratorClass == NULL) {
        return;
    }
    jmethodID hasNext = env->GetMethodID(iteratorClass, "hasNext", "()Z");
    if (hasNext == NULL) {
        return;
    }
    jmethodID next =
            env->GetMethodID(iteratorClass, "next", "()Ljava/lang/Object;");
    if (next == NULL) {
        return;
    }
    // Get the Entry class method IDs
    jclass entryClass = env->FindClass("java/util/Map$Entry");
    if (entryClass == NULL) {
        return;
    }
    jmethodID getKey =
            env->GetMethodID(entryClass, "getKey", "()Ljava/lang/Object;");
    if (getKey == NULL) {
        return;
    }
    jmethodID getValue =
            env->GetMethodID(entryClass, "getValue", "()Ljava/lang/Object;");
    if (getValue == NULL) {
        return;
    }
    // Iterate over the entry Set
    while (env->CallBooleanMethod(iter, hasNext)) {
        jobject entry = env->CallObjectMethod(iter, next);
        jstring key = (jstring) env->CallObjectMethod(entry, getKey);
        jstring value = (jstring) env->CallObjectMethod(entry, getValue);
        const char* keyStr = env->GetStringUTFChars(key, NULL);
        if (!keyStr) {  // Out of memory
            return;
        }
        const char* valueStr = env->GetStringUTFChars(value, NULL);
        if (!valueStr) {  // Out of memory
            env->ReleaseStringUTFChars(key, keyStr);
            return;
        }

        mapOut.insert(std::make_pair(std::string(keyStr), std::string(valueStr)));

        env->DeleteLocalRef(entry);
        env->ReleaseStringUTFChars(key, keyStr);
        env->DeleteLocalRef(key);
        env->ReleaseStringUTFChars(value, valueStr);
        env->DeleteLocalRef(value);
    }
}

JNIEXPORT jstring JNICALL Java_jp_co_drecom_util_config_AppConfigure_getWebViewErrorTemplate()
{
	JavaVM* _vm = cocos2d::JniHelper::getJavaVM();
	JNIEnv* _env;
	_vm->GetEnv((void**)&_env, JNI_VERSION_1_4);
	_vm->AttachCurrentThread(&_env, NULL);

	return _env->NewStringUTF(ANDROID_WEBVIEW_ERROR_TEMPLATE);
}

// _BQ_TMPL_CONFIGURE_FUNC_BEGIN
// _BQ_TMPL_CONFIGURE_FUNC_SNSAUTH_BEGIN
jstring Java_jp_co_drecom_util_config_AppConfigure_getUserId(JNIEnv* env, jobject javaThis) {
    long long user_id = UserDataBridge::getInstance()->getUserId();
    std::string user_id_str = UtilityForSakura::bigintToString(user_id);
    return env->NewStringUTF(user_id_str.c_str());
}

JNIEXPORT void JNICALL Java_jp_co_drecom_util_config_AppConfigure_callbackSnsAuthnication(JNIEnv* env, jobject javaThis, jstring jcode, jstring joperation, jstring jtoken, jstring jsns_account) {

  SnsMobileDataMigration* instance = SnsMobileDataMigration::getInstance();

  if (instance != NULL) {
    SnsMobileDataMigrationListener* listener = instance->getListener();
    if (listener != NULL) {
      const char* code  = env->GetStringUTFChars(jcode, 0);
      const char* operation_str  = env->GetStringUTFChars(joperation, 0);
      int operation = atoi( operation_str );
      const char* token = env->GetStringUTFChars(jtoken, 0);
      const char* sns_account = env->GetStringUTFChars(jsns_account, 0);
      listener->onCallBack(code, operation, token, sns_account);
      env->ReleaseStringUTFChars(jcode, code);
      env->ReleaseStringUTFChars(joperation, operation_str);
      env->ReleaseStringUTFChars(jtoken, token);
      env->ReleaseStringUTFChars(jsns_account, sns_account);
    }
  }
  return;
}
// _BQ_TMPL_CONFIGURE_FUNC_SNSAUTH_END
// _BQ_TMPL_CONFIGURE_FUNC_END

JNIEXPORT void JNICALL
Java_jp_co_drecom_util_config_AppConfigure_callbackBNIDLink(JNIEnv *env, jclass clazz,
                                                            jobject key_value_map) {
    std::map<std::string, std::string> queryMap;
    javaHashMapToStlStringStringMap(env, key_value_map, queryMap);
    auto listener = BNIDLinkResultChecker::getInstance()->getListener();
    if (listener != NULL) {
        listener->onCallBack(queryMap);
    }
    return;
}
}	//  extern "C"
